// Copyright (c) 1995  David Engberg  All rights reserved
// $Id: JavaConstant.h,v 1.5 1998/07/28 20:36:02 nickm Exp $
#ifndef _JavaConstant_h
#define _JavaConstant_h
#pragma interface

#include "unicode_string.h"
#include "dynamic_cast.h"
#include "config.h"

#ifndef OLDGCC
using namespace std;
#endif

class ostream;

// Java bytes are 8 bits.
#if (SIZEOF_CHAR == 1)
   typedef signed char     s8;
   typedef unsigned char   u8;
#else
   #error "Char is supposed to be 1 byte wide! (Stroustrup 2.3.1)"
#endif

// Java shorts and chars are 16 bits.
#if (SIZEOF_SHORT == 2)
   typedef unsigned short  u16;
   typedef signed short    s16;
#elif (SIZEOF_INT == 2)
   typedef unsigned int    u16;
   typedef signed int      s16
#elif (SIZEOF_LONG == 2)
   typedef unsigned long   u16;
   typedef signed long     s16;
#else
   #error "Neither short, int, or long is 2 bytes wide."
#endif

// Java ints are 32 bits.
#if (SIZEOF_INT == 4)
   typedef unsigned int        u32;
   typedef signed int          s32;
#elif (SIZEOF_LONG == 4)
   typedef unsigned long       u32;
   typedef signed long         s32;
#elif (SIZEOF_LONG_LONG == 4)
   typedef unsigned long long  u32;
   typedef signed long long    s32;
#else
   #error "Neither int, long, or long long is 4 bytes wide."
#endif

// Java longs are 64 bits.
#if (SIZEOF_INT == 8)
   typedef unsigned int        u64;
   typedef signed int          s64;
#elif (SIZEOF_LONG == 8)
   typedef unsigned long       u64;
   typedef signed long         s64;
#elif (SIZEOF_LONG_LONG == 8)
   typedef unsigned long long  u64;
   typedef signed long long    s64;
#else
   #error "Neither int, long, or long long is 8 bytes wide."
#endif

typedef u8  JavaUByte;
typedef s8  JavaSByte;
typedef u16 JavaUChar;
typedef u16 JavaUShort;
typedef s16 JavaSShort;
typedef u32 JavaUInt;
typedef s32 JavaSInt;
typedef u64 JavaULong;
typedef s64 JavaSLong;

typedef u16 JavaConstantIndex;

//
//  Class name : CJavaConstant
//  Description : This virtual base class is designed to represent a single
//    entry in a Java class's constant pool.  It doesn't actually do anything
//    in its own right other than set up the protocol for all of the other
//    constants.  When some function hands off a pointer to a CJavaConstant,
//    the user needs to use the DYNAMIC_CAST macro to figure out what the
//    actual constant is.  Messy?  yeah, so's the Java VM layout.
//
class CJavaConstant {
  DynamicCastDeclarations;
public:
  virtual ~CJavaConstant();
  
  typedef enum { kClass = 7,
		 kField = 9,
		 kMethod = 10,
		 kString = 8,
		 kInteger = 3,
		 kFloat = 4,
		 kLong = 5,
		 kDouble = 6,
		 kInterface = 11,
		 kNameType = 12,
		 kUnicode = 2,
		 kAsciz = 1 } ConstantType;
  virtual ConstantType GetConstantType() const = 0;
  virtual unsigned long GetWidth() const = 0;
  virtual void Disassemble(ostream& toStream) const = 0;
  virtual bool operator==(const CJavaConstant& other) const = 0;
};


//
//  Class name : CJavaClassConstant
//  Description : This represents a CONSTANT_Class value from the Java
//    class file format.
//
class CJavaClassConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaClassConstant(JavaConstantIndex nameIndex);
  virtual ~CJavaClassConstant();
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  virtual bool operator==(const CJavaConstant& other) const;
  JavaConstantIndex GetNameIndex() const { return fNameIndex; }
  virtual void Disassemble(ostream& toStream) const;
private:
  JavaConstantIndex fNameIndex;
};


//
//  Class name : CJavaFieldConstant
//  Description : This represents a CONSTANT_Fieldref value from the Java
//    class file format.
//
class CJavaFieldConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaFieldConstant(JavaConstantIndex classIndex,
		     JavaConstantIndex nameTypeIndex);
  virtual ~CJavaFieldConstant();
  virtual bool operator==(const CJavaConstant& other) const;
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  JavaConstantIndex GetClassIndex() const { return fClassIndex; }
  JavaConstantIndex GetNameTypeIndex() const { return fNameTypeIndex; }
  virtual void Disassemble(ostream& toStream) const;
private:
  JavaConstantIndex fClassIndex;
  JavaConstantIndex fNameTypeIndex;
};


//
//  Class name : CJavaMethodConstant
//  Description : This represents a CONSTANT_MethodConstant value from the Java
//    class file format.
//
class CJavaMethodConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaMethodConstant(JavaConstantIndex classIndex,
		      JavaConstantIndex nameTypeIndex);
  virtual ~CJavaMethodConstant();
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  virtual bool operator==(const CJavaConstant& other) const;
  JavaConstantIndex GetClassIndex() const { return fClassIndex; }
  JavaConstantIndex GetNameTypeIndex() const { return fNameTypeIndex; }
  virtual void Disassemble(ostream& toStream) const;
private:
  JavaConstantIndex fClassIndex;
  JavaConstantIndex fNameTypeIndex;
};


//
//  Class name : CJavaStringConstant
//  Description : This represents a CONSTANT_String value from the Java
//    class file format.
//
class CJavaStringConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaStringConstant(JavaConstantIndex stringIndex);
  virtual ~CJavaStringConstant();
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  virtual bool operator==(const CJavaConstant& other) const;
  JavaConstantIndex GetStringIndex() const { return fStringIndex; }
  virtual void Disassemble(ostream& toStream) const;
private:
  JavaConstantIndex fStringIndex;
};


//
//  Class name : CJavaIntegerConstant
//  Description : This represents a CONSTANT_Integer value from the Java
//    class file format.
//
class CJavaIntegerConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaIntegerConstant(JavaSInt number);
  virtual ~CJavaIntegerConstant();
  virtual bool operator==(const CJavaConstant& other) const;
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  long GetInteger() const { return fInteger; }
  virtual void Disassemble(ostream& toStream) const;
private:
  JavaSInt fInteger;
};


//
//  Class name : CJavaFloatConstant
//  Description : This represents a CONSTANT_Float value from the Java
//    class file format.
//
class CJavaFloatConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaFloatConstant(float number);
  virtual ~CJavaFloatConstant();
  virtual bool operator==(const CJavaConstant& other) const;
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  float GetFloat() const { return fFloat; }
  virtual void Disassemble(ostream& toStream) const;
private:
  float fFloat;
};


//
//  Class name : CJavaLongConstant
//  Description : This represents a CONSTANT_Long value from the Java
//    class file format.
//
class CJavaLongConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaLongConstant(JavaULong value);
  virtual ~CJavaLongConstant();
  virtual bool operator==(const CJavaConstant& other) const;
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  virtual void Disassemble(ostream& toStream) const;
  JavaSLong GetLong() const { return fLong; }
private:
  JavaULong fLong;
};


//
//  Class name : CJavaDoubleConstant
//  Description : This represents a CONSTANT_Double value from the Java
//    class file format.
//
class CJavaDoubleConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaDoubleConstant(double number);
  virtual ~CJavaDoubleConstant();
  virtual bool operator==(const CJavaConstant& other) const;
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  double GetDouble() const { return fDouble; }
  virtual void Disassemble(ostream& toStream) const;
private:
  double fDouble;
};


//
//  Class name : CJavaInterfaceConstant
//  Description : This represents a CONSTANT_InterfaceMethodref value from the
//    Java class file format.
//
class CJavaInterfaceConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaInterfaceConstant(JavaConstantIndex classIndex,
			 JavaConstantIndex nameTypeIndex);
  virtual ~CJavaInterfaceConstant();
  virtual bool operator==(const CJavaConstant& other) const;
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  JavaConstantIndex GetClassIndex() const { return fClassIndex; }
  JavaConstantIndex GetNameTypeIndex() const { return fNameTypeIndex; }
  virtual void Disassemble(ostream& toStream) const;
private:
  JavaConstantIndex fClassIndex;
  JavaConstantIndex fNameTypeIndex;
};


//
//  Class name : CJavaNameTypeConstant
//  Description : This represents a CONSTANT_NameandType value from the Java
//    class file format.
//
class CJavaNameTypeConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaNameTypeConstant(JavaConstantIndex nameIndex,
			JavaConstantIndex signatureIndex);
  virtual ~CJavaNameTypeConstant();
  virtual bool operator==(const CJavaConstant& other) const;
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  JavaConstantIndex GetNameIndex() const { return fNameIndex; }
  JavaConstantIndex GetSignatureIndex() const { return fSignatureIndex; }
  virtual void Disassemble(ostream& toStream) const;
private:
  JavaConstantIndex fNameIndex;
  JavaConstantIndex fSignatureIndex;
};


//
//  Class name : CJavaAscizConstant
//  Description : This represents a CONSTANT_Asciz value from the Java
//    class file format.
//
class CJavaAscizConstant : public CJavaConstant {
  DynamicCastDeclarations;
public:
  CJavaAscizConstant(const char* bytes, long length);
  CJavaAscizConstant(const unicode_string& unicodeConstant);
  virtual ~CJavaAscizConstant();
  virtual bool operator==(const CJavaConstant& other) const;
  virtual ConstantType GetConstantType() const;
  virtual unsigned long GetWidth() const;
  unicode_string GetUnicodeString() const { return fJavaString; }
  virtual void Disassemble(ostream& toStream) const;
private:
  unicode_string fJavaString;
};

#endif
