// Copyright (c) 1996  David Engberg  All rights reserved
// $Id: InstantiationList.h,v 1.6 1998/08/20 18:30:42 nickm Exp $
#ifndef _InstantiationList_h
#define _InstantiationList_h
#pragma interface

#include "JavaTypeSignature.h"
#include <set>
#include <functional>

class ostream;
class CCompiler;
class CSourceFile;

typedef set<CJavaTypeSignature, less<CJavaTypeSignature> > TypeSet;

//
//  Class name  : InstantiationList
//  Description : This class is used by the compiler to keep track of which
//      ITypes it needs to generate code for.  A simple set is insufficient,
//      since we want to keep from instantiating the same type twice.
//
class CInstantiationList {
public:
  CInstantiationList();
  CInstantiationList(const CInstantiationList& source);
  ~CInstantiationList();
  
  void AddType(const CJavaTypeSignature& type,
	       CSourceFile *file);
  void AddTypes(const TypeSet& types,
		CSourceFile *file);
  CInstantiationList &operator=(const CInstantiationList& source);

  bool Finished() const;
  const CJavaTypeSignature& StartImplementing();
  void WriteToStream(ostream& out) const;

protected:
  friend class CCompiler;
  void SetCompilerAlias(CCompiler* alias);

private:
  TypeSet fFinished;
  TypeSet fNotFinished;
  CCompiler *fCompilerAlias;
};

#endif // _InstantiationList_h
